@@ -45,7 +45,7 @@
  * Portions Copyright (c) 1994, Regents of the University of California
  *
  * IDENTIFICATION
- *	  $Header: /cvsroot/pgsql-server/src/backend/executor/nodeAgg.c,v 1.116 2003/08/19 01:13:40 tgl Exp $
+ *	  $Header: /cvsroot/pgsql/src/backend/executor/nodeAgg.c,v 1.116.2.2 2004/07/10 18:39:44 tgl Exp $
  *
  *-------------------------------------------------------------------------
  */
@@ -251,6 +251,18 @@ initialize_aggregates(AggState *aggstate,
 									  false);
 		}
 
+		/*
+		 * If we are reinitializing after a group boundary, we have to free
+		 * any prior transValue to avoid memory leakage.  We must check not
+		 * only the isnull flag but whether the pointer is NULL; since
+		 * pergroupstate is initialized with palloc0, the initial condition
+		 * has isnull = 0 and null pointer.
+		 */
+		if (!peraggstate->transtypeByVal &&
+			!pergroupstate->transValueIsNull &&
+			DatumGetPointer(pergroupstate->transValue) != NULL)
+			pfree(DatumGetPointer(pergroupstate->transValue));
+
 		/*
 		 * (Re)set transValue to the initial value.
 		 *
@@ -662,7 +674,6 @@ agg_retrieve_direct(AggState *aggstate)
 	AggStatePerGroup pergroup;
 	TupleTableSlot *outerslot;
 	TupleTableSlot *firstSlot;
-	TupleTableSlot *resultSlot;
 	int			aggno;
 
 	/*
@@ -684,11 +695,8 @@ agg_retrieve_direct(AggState *aggstate)
 	 * We loop retrieving groups until we find one matching
 	 * aggstate->ss.ps.qual
 	 */
-	do
+	while (!aggstate->agg_done)
 	{
-		if (aggstate->agg_done)
-			return NULL;
-
 		/*
 		 * If we don't already have the first tuple of the new group,
 		 * fetch it from the outer plan.
@@ -803,7 +811,7 @@ agg_retrieve_direct(AggState *aggstate)
 		/*
 		 * If we have no first tuple (ie, the outerPlan didn't return
 		 * anything), create a dummy all-nulls input tuple for use by
-		 * ExecProject. 99.44% of the time this is a waste of cycles,
+		 * ExecQual/ExecProject. 99.44% of the time this is a waste of cycles,
 		 * because ordinarily the projected output tuple's targetlist
 		 * cannot contain any direct (non-aggregated) references to input
 		 * columns, so the dummy tuple will not be referenced. However
@@ -845,22 +853,28 @@ agg_retrieve_direct(AggState *aggstate)
 		}
 
 		/*
-		 * Form a projection tuple using the aggregate results and the
-		 * representative input tuple.	Store it in the result tuple slot.
-		 * Note we do not support aggregates returning sets ...
+		 * Use the representative input tuple for any references to
+		 * non-aggregated input columns in the qual and tlist.
 		 */
 		econtext->ecxt_scantuple = firstSlot;
-		resultSlot = ExecProject(projInfo, NULL);
 
 		/*
-		 * If the completed tuple does not match the qualifications, it is
-		 * ignored and we loop back to try to process another group.
-		 * Otherwise, return the tuple.
+		 * Check the qual (HAVING clause); if the group does not match,
+		 * ignore it and loop back to try to process another group.
 		 */
+		if (ExecQual(aggstate->ss.ps.qual, econtext, false))
+		{
+			/*
+			 * Form and return a projection tuple using the aggregate results
+			 * and the representative input tuple.  Note we do not support
+			 * aggregates returning sets ...
+			 */
+			return ExecProject(projInfo, NULL);
+		}
 	}
-	while (!ExecQual(aggstate->ss.ps.qual, econtext, false));
 
-	return resultSlot;
+	/* No more groups */
+	return NULL;
 }
 
 /*
@@ -922,7 +936,6 @@ agg_retrieve_hash_table(AggState *aggstate)
 	AggStatePerGroup pergroup;
 	AggHashEntry entry;
 	TupleTableSlot *firstSlot;
-	TupleTableSlot *resultSlot;
 	int			aggno;
 
 	/*
@@ -940,11 +953,8 @@ agg_retrieve_hash_table(AggState *aggstate)
 	 * We loop retrieving groups until we find one satisfying
 	 * aggstate->ss.ps.qual
 	 */
-	do
+	while (!aggstate->agg_done)
 	{
-		if (aggstate->agg_done)
-			return NULL;
-
 		/*
 		 * Find the next entry in the hash table
 		 */
@@ -987,22 +997,28 @@ agg_retrieve_hash_table(AggState *aggstate)
 		}
 
 		/*
-		 * Form a projection tuple using the aggregate results and the
-		 * representative input tuple.	Store it in the result tuple slot.
-		 * Note we do not support aggregates returning sets ...
+		 * Use the representative input tuple for any references to
+		 * non-aggregated input columns in the qual and tlist.
 		 */
 		econtext->ecxt_scantuple = firstSlot;
-		resultSlot = ExecProject(projInfo, NULL);
 
 		/*
-		 * If the completed tuple does not match the qualifications, it is
-		 * ignored and we loop back to try to process another group.
-		 * Otherwise, return the tuple.
+		 * Check the qual (HAVING clause); if the group does not match,
+		 * ignore it and loop back to try to process another group.
 		 */
+		if (ExecQual(aggstate->ss.ps.qual, econtext, false))
+		{
+			/*
+			 * Form and return a projection tuple using the aggregate results
+			 * and the representative input tuple.  Note we do not support
+			 * aggregates returning sets ...
+			 */
+			return ExecProject(projInfo, NULL);
+		}
 	}
-	while (!ExecQual(aggstate->ss.ps.qual, econtext, false));
 
-	return resultSlot;
+	/* No more groups */
+	return NULL;
 }
 
 /* -----------------
@@ -1472,6 +1488,12 @@ ExecReScanAgg(AggState *node, ExprContext *exprCtxt)
 		build_hash_table(node);
 		node->table_filled = false;
 	}
+	else
+	{
+		/* Reset the per-group state (in particular, mark transvalues null) */
+		MemSet(node->pergroup, 0,
+			   sizeof(AggStatePerGroupData) * node->numaggs);
+	}
 
 	/*
 	 * if chgParam of subnode is not null then plan will be re-scanned by
